﻿/**
 *	MiniBatteryStatus Widget
 *
 *	© Claudio Procida 2005-2008
 *
 *	Disclaimer
 *
 *	The MiniBatteryStatus Widget software (from now, the "Software") and the accompanying materials
 *	are provided “AS IS” without warranty of any kind. IN NO EVENT SHALL THE AUTHOR(S) BE
 *	LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 *	INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 *	IF THE AUTHOR(S) HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. The entire risk as to
 *	the results and performance of this software is assumed by you. If the software is
 *	defective, you, and not its author(s), assume the entire cost of all necessary servicing,
 *	repairs and corrections. If you do not agree to these terms and conditions, you may not
 *	install or use this software.
 */


/**
 *	Constants
 */

var GAUGE_MAX = 100,
	GAUGE_MIN = 9,
	
	// Sizes
	
	FRONT_WIDTH = 147,
	FRONT_HEIGHT = 71,
	BACK_WIDTH = 258,
	BACK_HEIGHT = 161,
	
	// Notifications
	
	UPDATE_AVAILABLE = "Update Available",
	
	WIDGET_HOME_URL = "http://www.emeraldion.it/software/widgets/minibatterystatus.html";

/**
 *	Variables
 */

var currentCharge = 0,
    timeToEmpty = -1,
    timeToFullCharge = -1,
	maxCharge = 100,
	name = "",
	powerSourceState = "AC Power",
	
	previousCharge = 0,
	wasPlugged = -1,
	wasCharging = -1;

var view = "percentage",
	flipShown = false,
	animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null},
	batteryindex = 0,
	
	// Sizes
	
	currWidth = FRONT_WIDTH,
	currHeight = FRONT_HEIGHT,

	// Preferences

	checkUpdates = true,
	
	// Environment vars

	batteryCount = 0;	// Number of installed batteries

/**
 *	Parameters
 */

var views = ["time", "details", "none", "percentage"];
	
/**
 *	Init function
 */

function setup() {
	if (window.widget) {
	    widget.onshow = onshow;
	    widget.onhide = onhide;
	    widget.onremove = onremove;
	}
	loadEnvironment();
	readPreferences();
	loadUI();
	localizeUI();
	displayStatus();
}

function loadUI() {
	new AppleGlassButton(getObj("done"), getLocalizedString("Done"), hidePreferences);
	new AppleGlassButton(getObj("energysaver"), getLocalizedString("Energy Saver"), energySaver);
	
	var frm = getObj("prefFrm");
	for (var i = 0; i < batteryCount; i++) {
		frm.batteryIndex.options[i] = new Option(getLocalizedString("Battery #") + (i + 1), i);
	}
	frm.batteryIndex.options[batteryCount] = new Option(getLocalizedString("Demo"), batteryCount);
	
	getObj("version").innerHTML = "v" + trim(getWidgetProperty("CFBundleVersion"));

	if (checkUpdates) {
		var versionChecker = new VersionChecker();
		versionChecker.onUpdateCall(showAvailableUpdate);
		versionChecker.checkUpdate();
	}
}

function loadEnvironment() {
	batteryCount = Plugin.getBatteryCount();
}

/**
 *	Battery monitor functions
 */

function cycleView() {
	view = views.shift();
	views.push(view);
	widget.setPreferenceForKey(view, widget.identifier + "view");
	displayStatus();
}

function displayStatus() {

	if (!Plugin.isInstalled()) {
		showObj("nobattery");
		hideObj("value");
		hideObj("gauge");
		ArcoIris.update({plugged:Plugin.isPlugged()});
	}
	else
	{
		hideObj("nobattery");
		showObj("value");
		showObj("gauge");

		switch (view) {
			case "time":
				var timeToDisplay;
				var timeToFull = Plugin.timeToFull();
				var timeToEmpty = Plugin.timeToEmpty();
				if (timeToFull != -1 &&
					timeToEmpty != -1)
				{
					timeToDisplay = timeToEmpty ? timeToEmpty : timeToFull;
					getObj("time").innerHTML = seconds2time(timeToDisplay);
				}
				else
				{
					getObj("time").innerHTML = "∞";
				}
	
				hideObj("percent");
				hideObj("details");
				showObj("time");
				hideObj("none");
	
				break;
			case "percentage":
				getObj("percent").innerHTML = Plugin.charge() + "%";
	
				showObj("percent");
				hideObj("details");
				hideObj("time");
				hideObj("none");
	
				break;
			case "details":
			
				getObj("capacity").innerHTML = Plugin.maxCapacity() + "|" + Plugin.designCapacity();
				getObj("cycles").innerHTML = Plugin.cycleCount();
	
				hideObj("percent");
				showObj("details");
				hideObj("time");
				hideObj("none");
			
				break;
			default: // empty view
	
				hideObj("percent");
				hideObj("details");
				hideObj("time");
				showObj("none");
		}

		getObj("gauge").style.width = clampTo(Math.round(GAUGE_MAX * Plugin.charge() / 100), GAUGE_MIN, GAUGE_MAX) + 'px';
		
		if (Plugin.isPlugged())
		{
			if (Plugin.isCharging())
			{
				hideObj("plugged-icon");
				showObj("charging-icon");
			}
			else
			{
				showObj("plugged-icon");
				hideObj("charging-icon");
			}
			ArcoIris.update({plugged:true});
		}
		else
		{
			hideObj("plugged-icon");
			hideObj("charging-icon");
			ArcoIris.update({plugged:false, charge:Plugin.charge()});
		}
	}
	
	
	if (Plugin.isPlugged() &&
		!Plugin.isCharging() &&
		wasCharging == 1)
	{
		Growl.notify(GROWL_BATTERY_FULL_NOTIFICATION);
	}
	else if (previousCharge > Plugin.charge()) // Notify only if decreasing
	{
		Growl.notify(GROWL_BATTERY_LOW_NOTIFICATION, Plugin.charge());
	}
	if (wasPlugged != -1 &&
		(Plugin.isPlugged() && !wasPlugged ||
		 !Plugin.isPlugged() && wasPlugged))
	{
		powerSourceState = Plugin.isPlugged() ? "AC Power" : "Battery Power";
		Growl.notify(GROWL_POWER_CHANGED_NOTIFICATION, powerSourceState);
	}
	
	previousCharge = Plugin.charge();
	wasPlugged = Plugin.isPlugged();
	wasCharging = Plugin.isCharging();
}

/**
 *	Dashboard event handlers
 */

function onshow() {
	displayStatus(); // Instant feedback
}

function onhide() {

}

function onremove() {
	widget.setPreferenceForKey(null, widget.identifier + "batteryIndex");
}

function mousemove (event) {
	if (!flipShown) {
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}
 
		var starttime = (new Date).getTime() - 13;
 
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ("flip");
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}

function mouseexit (event) {
	if (flipShown) {
		// fade in the info button
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}

		var starttime = (new Date).getTime() - 13;

		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ("flip");
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

/**
 *	Preferences handling
 */

function readPreferences() {
	view = widget.preferenceForKey(widget.identifier + "view");
	if (view == null)
	{
		view = "percentage";
	}
	checkUpdates = widget.preferenceForKey("checkUpdates");
	batteryIndex = widget.preferenceForKey(widget.identifier + "batteryIndex");
	if (isNaN(batteryIndex)) {
		batteryIndex = 0;
	}
	Plugin.monitorBatteryAtIndex(batteryIndex);

	Growl.enabled = widget.preferenceForKey("enableGrowl");

	Growl.notifyBatteryFull = widget.preferenceForKey("notifyBatteryFull");
	Growl.notifyBatteryLow = widget.preferenceForKey("notifyBatteryLow");
	Growl.notifyPowerChange = widget.preferenceForKey("notifyPowerChange");
	Growl.notifyLowLevel = widget.preferenceForKey("notifyLowLevel");

	var frm = getObj("prefFrm");

	// General
	frm.checkUpdates.checked = checkUpdates;

	// Growl
	frm.enableGrowl.checked = Growl.enabled;

	frm.notifyBatteryFull.disabled = !Growl.enabled;
	frm.notifyBatteryFull.checked = Growl.notifyBatteryFull;

	frm.notifyBatteryLow.disabled = !Growl.enabled;
	frm.notifyBatteryLow.checked = Growl.notifyBatteryLow;

	frm.notifyPowerChange.disabled = !Growl.enabled;
	frm.notifyPowerChange.checked = Growl.notifyPowerChange;

	with (frm.notifyLowLevel) {
		selectedIndex = 0; // Lifesaver
		for (var i=0; i<options.length; i++) {
			if (options[i].value == Growl.notifyLowLevel)
				selectedIndex = i;
		}
	}
	frm.notifyLowLevel.disabled = !Growl.enabled || !Growl.notifyBatteryLow;

	// Sharing

	if (SharingService.enabled = widget.preferenceForKey("share"))
	{
		SharingService.start();
		Archive.sendData();
	}
	frm.share.checked = SharingService.enabled;
}

function savePreferences() {
	var frm = getObj("prefFrm");

	checkUpdates = frm.checkUpdates.checked;
	widget.setPreferenceForKey(checkUpdates, "checkUpdates");

	batteryIndex = frm.batteryIndex.selectedIndex;
	widget.setPreferenceForKey(batteryIndex, widget.identifier + "batteryIndex");

	Growl.enabled = frm.enableGrowl.checked;
	widget.setPreferenceForKey(Growl.enabled, "enableGrowl");

	Growl.notifyBatteryFull = frm.notifyBatteryFull.checked;
	widget.setPreferenceForKey(Growl.notifyBatteryFull, "notifyBatteryFull");

	Growl.notifyBatteryLow = frm.notifyBatteryLow.checked;
	widget.setPreferenceForKey(Growl.notifyBatteryLow, "notifyBatteryLow");

	Growl.notifyPowerChange = frm.notifyPowerChange.checked;
	widget.setPreferenceForKey(Growl.notifyPowerChange, "notifyPowerChange");

	Growl.notifyLowLevel = frm.notifyLowLevel.options[frm.notifyLowLevel.selectedIndex].value;
	widget.setPreferenceForKey(Growl.notifyLowLevel, "notifyLowLevel");
	
	SharingService.enabled = frm.share.checked;
	widget.setPreferenceForKey(SharingService.enabled , "share");
}

function showPreferences() {

	var front = getObj("front");
	var back = getObj("back");

	currWidth = BACK_WIDTH;
	currHeight = BACK_HEIGHT;
	window.resizeTo(currWidth, currHeight);

	if (window.widget)
		widget.prepareForTransition("ToBack");
       
	front.style.display="none";
	back.style.display="block";

	if (window.widget)
		setTimeout ("widget.performTransition();", 0);
}

function hidePreferences() {

	var front = getObj("front");
	var back = getObj("back");

	savePreferences();

	if (window.widget)
		widget.prepareForTransition("ToFront");

	back.style.display="none";
	front.style.display="block";

	currWidth = FRONT_WIDTH;
	currHeight = FRONT_HEIGHT;
	window.resizeTo(currWidth, currHeight);

	hidefliprollie();
	hideflip();

	if (window.widget)
		setTimeout ("widget.performTransition();", 0);
}

/**
 *	Visual FX
 */

function animate() {
	var T;
	var ease;
	var time = (new Date).getTime();
   

	T = limit_3(time-animation.starttime, 0, animation.duration);

	if (T >= animation.duration) {
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}

	animation.firstElement.style.opacity = animation.now;
}

/**
 *	Internationalization
 */

function localizeUI() {

	// Main view
	
	getObj("time").title = getLocalizedString("Remaining time");
	getObj("percent").title = getLocalizedString("Charge percentage");
	getObj("capacity").title = getLocalizedString("Current|Maximum capacity");
	getObj("cycles").title = getLocalizedString("Cycle count");

	getObj("charging-icon").title = getLocalizedString("Charging");
	getObj("plugged-icon").title = getLocalizedString("Plugged");


	// TabView buttons

	getObj("general-panel-selector").innerHTML = getLocalizedString("General");
	getObj("credits-panel-selector").innerHTML = getLocalizedString("Credits");
	getObj("sharing-panel-selector").innerHTML = getLocalizedString("Sharing");

	// General Preferences

	getObj("batteryIndexLabel").innerHTML = getLocalizedString("Monitor");
	getObj("checkUpdatesLabel").innerHTML = getLocalizedString("Check for updates");

	// Sharing Preferences

	getObj("shareLabel").innerHTML = getLocalizedString("Share battery data");
	getObj("shareStatusLabel").innerHTML = SharingService.enabled ?
		getLocalizedString("Service is running") :
		getLocalizedString("Service is not running");

	// Growl Preferences

	getObj("enableGrowlLabel").title = getLocalizedString("Enable Growl notifications");
	getObj("notifyBatteryFullLabel").title = getLocalizedString("Notify Battery Full");
	getObj("notifyBatteryLowLabel").title = getLocalizedString("Notify Battery Low");
	getObj("notifyBatteryLowLabel").innerHTML = getLocalizedString("from");
	getObj("notifyPowerChangeLabel").title = getLocalizedString("Notify Power Source changed");

	getObj("update").title = getLocalizedString("Update available");
	
	// Adjust elements sizes
	setTimeout(adjustUI, 100);
}

function adjustUI()
{
	// Reduce the width of tab titles to avoid overflow
	var titles = [
		getObj("general-panel-selector").innerHTML,
		getObj("credits-panel-selector").innerHTML,
		getObj("growl-panel-selector").innerHTML,
		getObj("sharing-panel-selector").innerHTML
	];
	
	var tabs = [
		getObj("general-panel-selector"),
		getObj("credits-panel-selector"),
		getObj("growl-panel-selector"),
		getObj("sharing-panel-selector")
	];

	// Reveal but keep invisible
	getObj("back").style.opacity = 0.0;
	getObj("back").style.display = "block";

	while (getObj("preferencepanel-toolbar").offsetHeight > 24)
	{
		var longest = titles.longest();
		var pos = titles.indexOf(longest);
		// Set up title for clipped labels
		if (!tabs[pos].getAttribute("title"))
		{
			tabs[pos].setAttribute("title", titles[pos]);
		}
		titles[pos] = longest.substring(0, longest.length - 1);
		tabs[pos].innerHTML = titles[pos] + "&hellip;";
	}
	// Hide again
	getObj("back").style.display = "none";
	getObj("back").style.opacity = 1.0;
}

function changeMonitoredBattery(sel)
{
	Plugin.monitorBatteryAtIndex(sel.selectedIndex);
	if (sel.selectedIndex < sel.options.length - 1) // Do not share fake demo data!
	{
		Archive.sendData();
	}
}

function changeShareStatus(chk)
{
	if (chk.checked)
	{
		SharingService.start();
	}
	else
	{
		SharingService.stop();
	}
}

/**
 *	Other handlers
 */

function energySaver() {
	var command = "/usr/bin/osascript Scripts/EnergySaver.scpt";
	widget.system(command, function()
		{
			widget.openApplication("com.apple.SystemPreferences");
		});
}

function showAvailableUpdate(updateAvailable, item, version) {
	if (updateAvailable) {
		showObj("update");
	}
}

/**
 *	Miscellaneous utilities
 */
 
function getObj(id) { // retrieves an element
	return document.getElementById(id);
}

function showObj(id) { // shows an element
	getObj(id).style.display = "block";
}

function hideObj(id) { // hides an element
	getObj(id).style.display = "none";
}

function threshold(value, minValue) {
	return Math.abs(value) < minValue ? 0 : value;
}

function clampTo(value, minValue, maxValue) {
	return value < minValue ? minValue : value > maxValue ? maxValue : value;
}

function padTo(num, len) {
	var s = String(num);
	while (s.length < len)
		s = '0' + s;
	return s;
}

function limit_3 (a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
	return from + (to - from) * ease;
}

function enterflip(event) {
	showfliprollie();
}

function exitflip(event) {
	hidefliprollie();
}

function showfliprollie() {
	getObj("fliprollie").style.display = 'block';
}
function hidefliprollie() {
	getObj("fliprollie").style.display = 'none';
}
function hideflip() {
	getObj("flip").style.opacity = 0;
}

function trim(str) {
	str = str.replace(/^\s+/, "");
	return str.replace(/\s+$/, "");
}

function goToUrl(url) {
	if (window.widget) {
		widget.openURL(url);
	}
	else {
		window.open(url);
	}
}

function selectPreferencePanel(selector) {
	var panels = document.getElementsByTagName("div");
	for (var i = 0; i < panels.length; i++) {
		if (panels[i].className == "preferencepanel") {
			if (selector.id.indexOf(panels[i].id) != -1) {
				showObj(panels[i].id);
			}
			else {
				hideObj(panels[i].id);
			}
		}
	}
	var anchors = document.getElementsByTagName("a");
	for (var i = 0; i < anchors.length; i++) {
		if (anchors[i].className.indexOf("tabviewbutton") == 0) {
			if (anchors[i] != selector && anchors[i].className.indexOf("selected") != -1) {
				anchors[i].className = anchors[i].className.substring(0, anchors[i].className.indexOf("-selected"));
			}
		}
	}

	if (selector.className.indexOf("selected") == -1) {
		selector.className += "-selected";
	}
}

function enableGrowlPreferences(control) {
	Growl.enabled = control.checked;
	
	var frm = getObj("prefFrm");

	frm.notifyBatteryFull.disabled = !Growl.enabled;
	frm.notifyBatteryLow.disabled = !Growl.enabled;
	frm.notifyPowerChange.disabled = !Growl.enabled;
	frm.notifyLowLevel.disabled = !(Growl.enabled && frm.notifyBatteryLow.checked);
}

function enableNotifyBatteryLow(control) {
	Growl.notifyBatteryLow = control.checked;
	
	var frm = getObj("prefFrm");

	frm.notifyLowLevel.disabled = !Growl.notifyBatteryLow;
}

function enableNotifyBatteryFull(control) {
	Growl.notifyBatteryFull = control.checked;
}

function enableNotifyPowerChange(control) {
	Growl.notifyPowerChange = control.checked;
}

function seconds2time(mins)
{
	var h = Math.floor(mins / 3600),
		m = Math.round((mins % 3600) / 60);
	return h + ":" + padTo(m, 2);
}

function __DEBUG(msg)
{
	var DEBUG_KEY = "Debug";
	if (widget.preferenceForKey(DEBUG_KEY))
	{
		alert(msg);
	}
}